//////////////////////////////////////////////////////////////////////
// Wave.cpp: implementation of the CWave class.
//
// This class implements an object that reads/writes a wave file.
//
//The following member functions are used to perform all the tasks:
//
//	UINT InOpen( CString* pPathName, WAVEFORMATEX* pWFX,
//					DWORD BufSize, PDWORD FileSize, DWORD offset);
//	LONG InRead( PSHORT pData, INT Length );
//	void InClose();
//
//	UINT OutOpen( CString* pPathName, WAVEFORMATEX* pWFX,
//						DWORD BufSize, DWORD SampleLimit, BOOL ChainFiles);
//	LONG OutWrite( PSHORT pData, INT Length  );
//	void OutClose();
//
//	LONGLONG GetDriveFreeSpace(CString* pPathName);
//
//////////////////////////////////////////////////////////////////////
// Copyright 1999.    Moe Wheatley AE4JY  <ae4jy@mindspring.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "IWave.h"
#include "Perform.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWave::CWave()
{
	m_INhmmio = FALSE;
	m_OUThmmio = FALSE;
	m_OutputOpen = FALSE;
	m_pInBuffer = NULL;
	m_pOutBuffer = NULL;
	m_InputOpen = FALSE;
	m_OutputOpen = FALSE;
	m_ChainFiles = 0;
	m_ChainCount = 0;
	InitializeCriticalSection(&m_CriticalSection);
}

CWave::~CWave()
{
	InClose();
	OutClose();
	DeleteCriticalSection(&m_CriticalSection);
}

//////////////////////////////////////////////////////////////////////
//  This function opens a wave file for reading.  It reads the wave file
//  information and places it in *pWFXSettings.  The file size in
//	number of samples is placed in *pFileSize. Number of samples can be
//  1,2 or 4 bytes long depending on bits per sample and number of channels.
//( ie. a 1000 sample buffer for 16 bit stereo will be a 4000 byte buffer)
//    parameters:
//		pPathName		= CString containing file path name.
//		pWFXSettings	= ptr to WAVEFORMATEX structure to put wavefile settings in.
//		BufSize			= DWORD specifies the wave file buffer size in
//							number of samples.  If the file ends before a 
//							buffer is completed, it is padded with 0's.
//						  If this value is Zero, the WAVEFORMATEX structure is
//							filled in and then the file is closed. This is useful
//							for just getting file info.
//		pFileSize		= pointer where the file length in number of samples is
//							written to.
//		Offset			= Number of samples into the file to start the reading
//								-1 == Cont loop mode
//    returns:
//        0			if opened OK
//    ErrorCode		if not
//////////////////////////////////////////////////////////////////////
UINT CWave::InOpen( CString* pPathName, WAVEFORMATEX* pWFX, DWORD BufSize,
							PDWORD pFileSize, DWORD Offset, AUXINFO* pAuxInfo, DWORD* pSource)
{
LONG dwFmtSize;
MMCKINFO mmckinSubchunk;
MMCKINFO mmckinfoParent;
DWORD Source = SOURCE_UNKNOWN;
	m_ContLoopMode = FALSE;
	m_ErrorCode = NO_ERRORS;
	m_pInBuffer = NULL;
	m_InBufPosition = 0;
	m_InputOpen = FALSE;
	m_InBlocksRead = 0;
	pAuxInfo->ADFrequency = 0;
	pAuxInfo->CenterFreq = 0;
	pAuxInfo->Bandwidth = 0;
	pAuxInfo->StartTime.wYear = 0;
	pAuxInfo->StopTime.wYear = 0;
	pAuxInfo->IFFrequency = 0;
	pAuxInfo->IQOffset = 0;
	pAuxInfo->DBOffset = 0;
	pAuxInfo->Unused3 = 0;
	pAuxInfo->Unused4 = 0;
	pAuxInfo->Unused5 = 0;
	pAuxInfo->Unused6 = 0;
	m_StartCtime = CTime::GetCurrentTime();;
	m_StopCtime = m_StartCtime + 100;
	memcpy(&m_FileStartTime, &pAuxInfo->StartTime, sizeof(SYSTEMTIME));

	
	CFile file;
	CFileStatus fstat;
	if( file.Open( (LPSTR)(LPCTSTR(*pPathName)),CFile::modeRead) )
	{
		file.GetStatus( fstat);
		file.Close();
	}
	
	if( (INT)Offset == -1)
	{
		m_ContLoopMode = TRUE;
		Offset = 0;
	}
	if( !(m_INhmmio = mmioOpen( (LPSTR)(LPCTSTR(*pPathName)), NULL,
									MMIO_READ|MMIO_ALLOCBUF)))
	{
		m_ErrorCode = WAVIN_ERR_OPEN;
	}
	else					//traverse the RIFF header to get info on file
	{
		mmckinfoParent.fccType = mmioFOURCC('W', 'A','V','E');
		if( mmioDescend( m_INhmmio, &mmckinfoParent, NULL,MMIO_FINDRIFF))
		{
			mmioClose(m_INhmmio,0);
			m_ErrorCode = WAVIN_ERR_NOTWAVE;
		}
		else
		{
			mmckinSubchunk.ckid = mmioFOURCC('f', 'm','t',' ');
			if( mmioDescend( m_INhmmio, &mmckinSubchunk,
					&mmckinfoParent,MMIO_FINDCHUNK))
			{
				mmioClose(m_INhmmio,0);
				m_ErrorCode = WAVIN_ERR_INVALID;
			}
			else
			{
				dwFmtSize = mmckinSubchunk.cksize;
				if( mmioRead( m_INhmmio, (HPSTR) &m_InFormat, dwFmtSize)
								!= dwFmtSize)
				{
					mmioClose(m_INhmmio,0);
					m_ErrorCode = WAVIN_ERR_INVALID;
				}
				else
				{
					mmioAscend( m_INhmmio, &mmckinSubchunk, 0);
					LONG pos = mmioSeek( m_INhmmio, 0, SEEK_CUR );
					if(pAuxInfo!=NULL)
					{
						mmckinSubchunk.ckid = mmioFOURCC('a','u','x','i');
						if( !mmioDescend( m_INhmmio, &mmckinSubchunk,
								&mmckinfoParent,MMIO_FINDCHUNK))
						{
							Source = SOURCE_SV;
							LONG ret;
							// read the various file versions around
							AUX1INFO oldinfo1;
							AUX2INFO oldinfo2;
							AUX3INFO oldinfo3;
							AUXWINRAD WinradInfo;
							switch(mmckinSubchunk.cksize)
							{
								case (sizeof(AUXINFO)):	//SV 3.0 and later
									ret = mmioRead( m_INhmmio, (HPSTR)pAuxInfo, mmckinSubchunk.cksize);
									break;
								case (sizeof(AUX3INFO)):	//SV 2.34 and earlier with local time bug
									ret = mmioRead( m_INhmmio, (HPSTR)&oldinfo3, mmckinSubchunk.cksize);
									pAuxInfo->ADFrequency = oldinfo3.ADFrequency;
									pAuxInfo->Bandwidth = oldinfo3.Bandwidth;
									pAuxInfo->CenterFreq = oldinfo3.CenterFreq;
									pAuxInfo->IFFrequency = oldinfo3.IFFrequency;
									pAuxInfo->IQOffset = oldinfo3.IQOffset;
									pAuxInfo->DBOffset = oldinfo3.DBOffset;
									pAuxInfo->Unused3 = oldinfo3.Unused3;
									pAuxInfo->Unused4 = oldinfo3.Unused4;
									pAuxInfo->Unused5 = oldinfo3.Unused5;
									pAuxInfo->Unused6 = 0;
									pAuxInfo->StartTime = ConvertLocalToUTC(&oldinfo3.StartTime);	//local time so need to
									pAuxInfo->StopTime = ConvertLocalToUTC(&oldinfo3.StopTime);		//convert to UTC
									break;
								case (sizeof(AUX2INFO)):
									ret = mmioRead( m_INhmmio, (HPSTR)&oldinfo2, mmckinSubchunk.cksize);
									pAuxInfo->ADFrequency = oldinfo2.ADFrequency;
									pAuxInfo->Bandwidth = oldinfo2.Bandwidth;
									pAuxInfo->CenterFreq = oldinfo2.CenterFreq;
									pAuxInfo->StartTime = oldinfo2.StartTime;
									pAuxInfo->StopTime = oldinfo2.StopTime;
									pAuxInfo->IFFrequency = 0;
									pAuxInfo->IQOffset = 0;
									pAuxInfo->DBOffset = 0;
									pAuxInfo->Unused3 = 0;
									pAuxInfo->Unused4 = 0;
									pAuxInfo->Unused5 = 0;
									break;
								case (sizeof(AUX1INFO)):
									ret = mmioRead( m_INhmmio, (HPSTR)&oldinfo1, mmckinSubchunk.cksize);
									pAuxInfo->ADFrequency = oldinfo1.ADFrequency;
									pAuxInfo->Bandwidth = oldinfo1.Bandwidth;
									pAuxInfo->CenterFreq = oldinfo1.CenterFreq;
									pAuxInfo->StartTime = oldinfo1.StartTime;
									pAuxInfo->StopTime = oldinfo1.StartTime;
									pAuxInfo->IFFrequency = 0;
									pAuxInfo->IQOffset = 0;
									pAuxInfo->DBOffset = 0;
									pAuxInfo->Unused3 = 0;
									pAuxInfo->Unused4 = 0;
									pAuxInfo->Unused5 = 0;
									break;
								case (sizeof(AUXWINRAD)):
									ret = mmioRead( m_INhmmio, (HPSTR)&WinradInfo, mmckinSubchunk.cksize);
									pAuxInfo->CenterFreq = WinradInfo.CenterFreq;
									pAuxInfo->ADFrequency = 66666667;
									pAuxInfo->Bandwidth = m_InFormat.nSamplesPerSec;
									pAuxInfo->StartTime = WinradInfo.StartTime;
									pAuxInfo->StopTime = WinradInfo.StopTime;
									pAuxInfo->IFFrequency = 0;
									pAuxInfo->IQOffset = 0;
									pAuxInfo->DBOffset = 0;
									pAuxInfo->Unused3 = 0;
									pAuxInfo->Unused4 = 0;
									pAuxInfo->Unused5 = 0;
									Source = SOURCE_WINRAD;
									break;
							}
							if( ret != (LONG)mmckinSubchunk.cksize)
							{
								mmioClose(m_INhmmio,0);
								m_ErrorCode = WAVIN_ERR_INVALID;
							}
							else
							{
								memcpy(&m_FileStartTime, &pAuxInfo->StartTime, sizeof(SYSTEMTIME));
								mmioAscend( m_INhmmio, &mmckinSubchunk, 0);
							}
						}
						else	//see if Perseus format wav file
						{
							mmioSeek( m_INhmmio, pos, SEEK_SET );
							mmckinSubchunk.ckid = mmioFOURCC('r','c','v','r');
							if( !mmioDescend( m_INhmmio, &mmckinSubchunk,
									&mmckinfoParent,MMIO_FINDCHUNK))
							{
								LONG ret;
								ret = mmioRead( m_INhmmio, (HPSTR)&m_PerseusHdr, mmckinSubchunk.cksize);
								if( ret != (LONG)mmckinSubchunk.cksize)
								{
									mmioClose(m_INhmmio,0);
									m_ErrorCode = WAVIN_ERR_INVALID;
								}
								else	//is Perseus file format
								{
									pAuxInfo->ADFrequency = 80000000;
									switch(m_PerseusHdr.SamplingRateIdx)
									{
										case 0:		//125KHz
											pAuxInfo->Bandwidth = 125000;
											break;
										case 1:		//250KHz
											pAuxInfo->Bandwidth = 250000;
											break;
										case 2:		//500KHz
											pAuxInfo->Bandwidth = 500000;
											break;
										case 3:		//1000KHz
											pAuxInfo->Bandwidth = 1000000;
											break;
										case 4:		//2000KHz
											pAuxInfo->Bandwidth = 2000000;
											break;
										default:
											pAuxInfo->Bandwidth = 125000;
											break;
									}
									pAuxInfo->CenterFreq = m_PerseusHdr.nCenterFrequencyHz;
									struct tm *t_tm;
									/* Obtain coordinated universal time: */
									t_tm = gmtime( &m_PerseusHdr.timeStart );
									pAuxInfo->StartTime.wDay = t_tm->tm_wday;
									pAuxInfo->StartTime.wDayOfWeek = t_tm->tm_wday;
									pAuxInfo->StartTime.wHour = t_tm->tm_hour;
									pAuxInfo->StartTime.wMinute = t_tm->tm_min;
									pAuxInfo->StartTime.wMonth = t_tm->tm_mon+1;
									pAuxInfo->StartTime.wSecond = t_tm->tm_sec;
									pAuxInfo->StartTime.wYear = t_tm->tm_year + 1900;
									pAuxInfo->StartTime.wMilliseconds = 0;
									pAuxInfo->StartTime.wDay = t_tm->tm_mday;

									pAuxInfo->StopTime = pAuxInfo->StartTime;	//gets changed later when file size is calculated
									pAuxInfo->IFFrequency = 0;
									pAuxInfo->IQOffset = 0;
									pAuxInfo->DBOffset = 0;	//attenuator??
									pAuxInfo->Unused3 = 0;
									pAuxInfo->Unused4 = 0;
									pAuxInfo->Unused5 = 0;
									Source = SOURCE_PERSEUS;
									mmioAscend( m_INhmmio, &mmckinSubchunk, 0);
								}
							}
							else
							{
								mmioSeek( m_INhmmio, pos, SEEK_SET );
							}
						}
					}
					mmckinSubchunk.ckid = mmioFOURCC('d', 'a','t','a');
					if( mmioDescend( m_INhmmio, &mmckinSubchunk, 
							     &mmckinfoParent,MMIO_FINDCHUNK))
					{
						mmioClose(m_INhmmio,0);
						m_ErrorCode = WAVIN_ERR_INVALID;
					}
					else
					{
						*pFileSize = mmckinSubchunk.cksize;
						if ( *pFileSize == 0L )
						{
							m_ErrorCode = WAVIN_ERR_NODATA;
							mmioClose(m_INhmmio,0);
						}
						else
						{
							if(  ( (m_InFormat.wFormatTag != WAVE_FORMAT_PCM )	&& ( m_InFormat.wFormatTag != WAVE_FORMAT_IEEE_FLOAT ) )
									|| ( (m_InFormat.nChannels != 1) &&
										 (m_InFormat.nChannels != 2) )
									|| (  (m_InFormat.wBitsPerSample != 8  ) &&
										  (m_InFormat.wBitsPerSample != 16 ) &&
										  (m_InFormat.wBitsPerSample != 24 ) &&
									      (m_InFormat.wBitsPerSample != 32 ) ) )
							{
								m_ErrorCode = WAVIN_ERR_NOTSUPPORTED;
								mmioClose(m_INhmmio,0);
							}
							else
							{
								if( ( m_InFormat.nSamplesPerSec < 1000 ) ||
									( m_InFormat.nSamplesPerSec > 100000000 ) )
								{
									m_ErrorCode = WAVIN_ERR_NOTSUPPORTED;
									mmioClose(m_INhmmio,0);
								}
							}
						}
					}
				}
			}
		}
	}
	if( m_ErrorCode == NO_ERRORS )
	{
		//  write file settings to the "pWFX" structure of the caller.
		pWFX->wFormatTag = m_InFormat.wFormatTag;
		pWFX->nChannels = m_InFormat.nChannels;
		pWFX->nSamplesPerSec = m_InFormat.nSamplesPerSec;
		pWFX->nAvgBytesPerSec = m_InFormat.nAvgBytesPerSec;
		pWFX->nBlockAlign = m_InFormat.nBlockAlign;
		pWFX->wBitsPerSample = m_InFormat.wBitsPerSample;
		m_InBytesPerSample = (m_InFormat.wBitsPerSample/8)*m_InFormat.nChannels;
		*pFileSize = *pFileSize / m_InBytesPerSample;//get # of samples

		if( *pFileSize > Offset )
		{
			mmioSeek( m_INhmmio, Offset*m_InFormat.nChannels*m_InBytesPerSample, SEEK_CUR );
			*pFileSize -= Offset;
		}
		// FileSize must be integer number of BufSizes due to padding
		if( BufSize != 0 )  // if not a file check
		{
			*pFileSize += ( BufSize - (*pFileSize%BufSize) );
			m_FileBlockSize = *pFileSize/BufSize;
			m_DataStartPos = mmioSeek( m_INhmmio, 0, SEEK_CUR );
			// allocate intermediate byte buffer
			m_InBufferSize = BufSize * m_InBytesPerSample;
			if( m_pInBuffer )
			{
				delete m_pInBuffer;
				m_pInBuffer = NULL;
			}
			m_pInBuffer = new char[ m_InBufferSize ];
			if( m_pInBuffer == NULL )
			{
				InClose();
				m_ErrorCode = MEMORY_ERROR;
			}
			else
			{
				m_InputOpen = TRUE;
			}
		}
		if(pAuxInfo->StartTime.wYear >2000 )
		{
			m_StartCtime = pAuxInfo->StartTime;
			m_StopCtime = pAuxInfo->StopTime;
			time_t filetime = (time_t)m_StopCtime.GetTime() - (time_t)m_StartCtime.GetTime();
			if(filetime>0)
			{	//is SpectraVue or Winrad format wav file
				m_BytesPerSecond = (*pFileSize*m_InBytesPerSample)/(DWORD)filetime;
				m_FileTotalSecs = (DWORD)filetime;
			}
			else
			{	//is Perseus format wav file
				m_BytesPerSecond = pWFX->nSamplesPerSec*m_InBytesPerSample;
				float tmp = (float)*pFileSize/(float)pWFX->nSamplesPerSec + (float)0.5;
				m_FileTotalSecs = (UINT)tmp;
				CTime stime(m_StartCtime);	//create CTime object == start time;
				time_t etm = stime.GetTime() + (time_t)m_FileTotalSecs;
				m_StopCtime = etm;
				m_StopCtime.GetAsSystemTime(pAuxInfo->StopTime);
			}
		}
		else
		{	//is Unknown source format wav file maybe PowerSDR
			m_BytesPerSecond = pWFX->nSamplesPerSec*m_InBytesPerSample;
			m_FileTotalSecs = (*pFileSize*m_InBytesPerSample)/m_BytesPerSecond;
			m_StartCtime = fstat.m_ctime.GetTime();
			m_StopCtime = m_StartCtime + m_FileTotalSecs;
			m_StartCtime.GetAsSystemTime( pAuxInfo->StartTime );
			m_StopCtime.GetAsSystemTime( pAuxInfo->StopTime );
		}
//TRACE("\n\r startTime = %d", m_StartCtime.GetSecond());
//TRACE("\n\r stopTime = %d", m_StopCtime.GetSecond());
		if(m_BytesPerSecond==0)
			m_BytesPerSecond = 1;
	}
	else
	{
		Source = SOURCE_UNKNOWN;
	}
	if( BufSize == 0 )  // see if just a file check
		InClose();	// if so close the input file
	if(pSource != NULL)
		*pSource = Source;
	return(m_ErrorCode);
}

///////////////////////////////////////////////////////////////////////
// Reads 'Length' samples of SHORT data from the opened wavefile.
//		pData	= Pointer to SHORT data array to be filled.
//		Length	= Number of samples to read into pData.
//		progress = Percentage of file completely read
//
//	returns:
//		Length	if 'Length' samples were succesfully read.
//		0  =	if reaches the end of the file.
//		-1 =	if there is an error ( use GetError() to retrieve error )
///////////////////////////////////////////////////////////////////////
LONG CWave::InRead( SHORT* pData , INT Length, DWORD& Progress )
{
INT i;
LONG numret;
	if( !m_InputOpen )
	{
		InClose();
		m_ErrorCode = WAVIN_ERR_NOTOPEN;
		return -1;
	}
	if( m_InBufPosition == 0 )	//if need to read in new buffer's worth
	{
		EnterCriticalSection(&m_CriticalSection);
		m_InBlocksRead++;
		Progress = (1000*m_InBlocksRead)/m_FileBlockSize;
//StartPerformance();
		numret = mmioRead( m_INhmmio, m_pInBuffer, m_InBufferSize );
//StopPerformance();
		LeaveCriticalSection(&m_CriticalSection);
		m_InBufferEndPos = m_InBufferSize;
		if( numret < 0 )		//read error
		{
			InClose();
			m_ErrorCode = WAVIN_ERR_READING;
			return -1;
		}
		else
		{
			if( numret == 0 )		//end of file
			{
				if( m_ContLoopMode )
				{
					long x=m_DataStartPos = mmioSeek( m_INhmmio, m_DataStartPos, SEEK_SET );
					numret = mmioRead( m_INhmmio, m_pInBuffer, m_InBufferSize );
					m_InBlocksRead = 1;
				}
				else
				{
					InClose();
					Progress = 999;
					return 0;
				}
			}
			else
			{
				if( numret < m_InBufferSize )	// got partial buffer worth from file
				{
					m_InBufPosition += numret;
					m_InBufPosition &= 0xFFFFFFFC;
					if( m_ContLoopMode )
					{
						long x = mmioSeek( m_INhmmio, m_DataStartPos, SEEK_SET );
						i = m_InBufferSize-m_InBufPosition;
						if( i>0 )
						{	//if a partial buffer left, then read enough to fill it
							numret = mmioRead( m_INhmmio, m_pInBuffer+m_InBufPosition, i );
							EnterCriticalSection(&m_CriticalSection);
							m_InBlocksRead = 0;
							Progress = (1000*m_InBlocksRead)/m_FileBlockSize;
							LeaveCriticalSection(&m_CriticalSection);
						}
					}
					else	//not continuous so pad remaining buffer
					{
						m_InBufferEndPos = m_InBufPosition;
						while( m_InBufPosition<m_InBufferSize)
						{
							if(m_InFormat.wBitsPerSample == 8)
								*(m_pInBuffer+m_InBufPosition) = (char)127;
							else
								*(m_pInBuffer+m_InBufPosition) = 0;
							m_InBufPosition++;
						}
					}
				}
			}
		}
		m_InBufPosition = 0;
	}
	if(m_InFormat.wBitsPerSample == 8)
	{
		m_usTemp.bytes.lsb = 0;
		for( i=0; i < (Length*m_InFormat.nChannels); i++ )
		{
			m_usTemp.bytes.msb = *(m_pInBuffer + (m_InBufPosition++) ) - 128;
			*(pData+i) = m_usTemp.both;
		}
	}
	else if(m_InFormat.wBitsPerSample == 16)
	{
		for( i=0; i < (Length*m_InFormat.nChannels); i++ )
		{
			m_usTemp.bytes.lsb = *(m_pInBuffer + (m_InBufPosition++) );
			m_usTemp.bytes.msb = *(m_pInBuffer + (m_InBufPosition++) );
			*( pData + i) = m_usTemp.both;
		}
	}
	else	//is 32 bit IEEE float format
	{
		for( i=0; i < (Length*m_InFormat.nChannels); i++ )
		{
			m_ufTemp.bytes.b0 = *(m_pInBuffer + (m_InBufPosition++) );
			m_ufTemp.bytes.b1 = *(m_pInBuffer + (m_InBufPosition++) );
			m_ufTemp.bytes.b2 = *(m_pInBuffer + (m_InBufPosition++) );
			m_ufTemp.bytes.b3 = *(m_pInBuffer + (m_InBufPosition++) );
			*( pData + i) = (SHORT)(m_ufTemp.all*32768.0);
		}
	}

	if( m_InBufPosition >= m_InBufferEndPos )
		m_InBufPosition = 0;	// need to read another buffer's worth
	return(Length);
}

///////////////////////////////////////////////////////////////////////
// Reads 'Length' samples of __int32 data from the opened wavefile.
//		pData	= Pointer to __int32 data array to be filled.
//		Length	= Number of samples to read into pData.
//		progress = Percentage of file completely read
//
//	returns:
//		Length	if 'Length' samples were succesfully read.
//		0  =	if reaches the end of the file.
//		-1 =	if there is an error ( use GetError() to retrieve error )
///////////////////////////////////////////////////////////////////////
LONG CWave::InRead( _int32* pData , INT Length, DWORD& Progress )
{
INT i;
LONG numret;
	if( !m_InputOpen )
	{
		InClose();
		m_ErrorCode = WAVIN_ERR_NOTOPEN;
		return -1;
	}
	if( m_InBufPosition == 0 )	//if need to read in new buffer's worth
	{
		EnterCriticalSection(&m_CriticalSection);
		m_InBlocksRead++;
		Progress = (1000*m_InBlocksRead)/m_FileBlockSize;
		numret = mmioRead( m_INhmmio, m_pInBuffer, m_InBufferSize );
		LeaveCriticalSection(&m_CriticalSection);
		m_InBufferEndPos = m_InBufferSize;
		if( numret < 0 )		//read error
		{
			InClose();
			m_ErrorCode = WAVIN_ERR_READING;
			return -1;
		}
		else
		{
			if( numret == 0 )		//end of file
			{
				if( m_ContLoopMode )
				{
					long x=m_DataStartPos = mmioSeek( m_INhmmio, m_DataStartPos, SEEK_SET );
					numret = mmioRead( m_INhmmio, m_pInBuffer, m_InBufferSize );
					m_InBlocksRead = 1;
				}
				else
				{
					InClose();
					Progress = 999;
					return 0;
				}
			}
			else
			{
				if( numret < m_InBufferSize )	// got partial buffer worth from file
				{
					m_InBufPosition += numret;
					m_InBufPosition &= 0xFFFFFFFC;
					if( m_ContLoopMode )
					{
						long x = mmioSeek( m_INhmmio, m_DataStartPos, SEEK_SET );
						i = m_InBufferSize-m_InBufPosition;
						if( i>0 )
						{	//if a partial buffer left, then read enough to fill it
							numret = mmioRead( m_INhmmio, m_pInBuffer+m_InBufPosition, i );
							EnterCriticalSection(&m_CriticalSection);
							m_InBlocksRead = 0;
							Progress = (1000*m_InBlocksRead)/m_FileBlockSize;
							LeaveCriticalSection(&m_CriticalSection);
						}
					}
					else	//not continuous so pad remaining buffer
					{
						m_InBufferEndPos = m_InBufPosition;
						while( m_InBufPosition<m_InBufferSize)
						{
							if(m_InFormat.wBitsPerSample == 8)
								*(m_pInBuffer+m_InBufPosition) = (char)127;
							else
								*(m_pInBuffer+m_InBufPosition) = 0;
							m_InBufPosition++;
						}
					}
				}
			}
		}
		m_InBufPosition = 0;
	}
	if(m_InFormat.wBitsPerSample == 24)
	{
		m_uiTemp.bytes.b0 = 0;
		for( i=0; i < (Length*m_InFormat.nChannels); i++ )
		{
			m_uiTemp.bytes.b1 = *(m_pInBuffer + (m_InBufPosition++) );
			m_uiTemp.bytes.b2 = *(m_pInBuffer + (m_InBufPosition++) );
			m_uiTemp.bytes.b3 = *(m_pInBuffer + (m_InBufPosition++) );
			*( pData + i) = m_uiTemp.all;
		}
	}
	else if(m_InFormat.wBitsPerSample == 16)
	{
		m_uiTemp.bytes.b0 = 0;
		m_uiTemp.bytes.b1 = 0;
		for( i=0; i < (Length*m_InFormat.nChannels); i++ )
		{
			m_uiTemp.bytes.b2 = *(m_pInBuffer + (m_InBufPosition++) );
			m_uiTemp.bytes.b3 = *(m_pInBuffer + (m_InBufPosition++) );
			*( pData + i) = m_uiTemp.all;
		}
	}
	else	//is 32 bit IEEE float format
	{
		for( i=0; i < (Length*m_InFormat.nChannels); i++ )
		{
			m_ufTemp.bytes.b0 = *(m_pInBuffer + (m_InBufPosition++) );
			m_ufTemp.bytes.b1 = *(m_pInBuffer + (m_InBufPosition++) );
			m_ufTemp.bytes.b2 = *(m_pInBuffer + (m_InBufPosition++) );
			m_ufTemp.bytes.b3 = *(m_pInBuffer + (m_InBufPosition++) );
			*( pData + i) = (__int32)(m_ufTemp.all*2.14748e9);
		}
	}

	if( m_InBufPosition >= m_InBufferEndPos )
		m_InBufPosition = 0;	// need to read another buffer's worth
	return(Length);
}


//////////////////////////////////////////////////////////////////////
// Changes current file position forwards or backwards.
// position is a value 0==start, 1000==end. (.1% steps)
//////////////////////////////////////////////////////////////////////
BOOL CWave::ChangeFilePosition(INT position)
{
INT blocks = (position*m_FileBlockSize)/1000;
INT bytes = m_InBufferSize*blocks;
int pad = bytes%m_InBytesPerSample;
	bytes -= pad;
	EnterCriticalSection(&m_CriticalSection);
	mmioSeek( m_INhmmio, m_DataStartPos+bytes, SEEK_SET);
	m_InBlocksRead = blocks;
	LeaveCriticalSection(&m_CriticalSection);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// sets the current file position in seconds from start
//////////////////////////////////////////////////////////////////////
void CWave::SetInPositionSecs(DWORD secs)
{
INT bytes = secs*m_BytesPerSecond;
int pad = bytes%m_InBytesPerSample;
	bytes -= pad;
	EnterCriticalSection(&m_CriticalSection);
	mmioSeek( m_INhmmio, m_DataStartPos+bytes, SEEK_SET);
	m_InBlocksRead = bytes/m_InBufferSize;
	LeaveCriticalSection(&m_CriticalSection);
}

//////////////////////////////////////////////////////////////////////
// returns the current file position in seconds from start
//////////////////////////////////////////////////////////////////////
DWORD CWave::GetInPositionSecs()
{
DWORD t;
	t = ((m_InBlocksRead*m_InBufferSize)/m_BytesPerSecond);
	return t;
}

//////////////////////////////////////////////////////////////////////
// returns the current file position in time
//////////////////////////////////////////////////////////////////////
time_t CWave:: GetInFilePosition()
{
time_t t;
	EnterCriticalSection(&m_CriticalSection);
	t = (time_t)((m_InBlocksRead*m_InBufferSize)/m_BytesPerSecond);
	LeaveCriticalSection(&m_CriticalSection);
//	if(m_FileStartTime.wYear > 2000)
	{
		t += (time_t)m_StartCtime.GetTime();
	}

	return t;
}

//////////////////////////////////////////////////////////////////////
//  Closes the Input wave file if open.
//
//////////////////////////////////////////////////////////////////////
void CWave::InClose()
{
	if( m_INhmmio )
	{
		mmioClose( m_INhmmio, 0 );
		m_INhmmio = NULL;
	}
	if( m_pInBuffer )
	{
		delete m_pInBuffer;
		m_pInBuffer = NULL;
	}
	m_InputOpen = FALSE;
}

//////////////////////////////////////////////////////////////////////
//  This function opens a wave file for writing.
//		Number of samples can be 1,2 or 4 bytes long depending on
//      bits per sample and number of channels.
//( ie. a 1000 sample buffer for 16 bit stereo will be a 4000 byte buffer)
//    parameters:
//		pPathName	= CString containing file path name
//		pWFX		= WAVEFORMATEX structure with desired wavefile settings
//		BufSize		= DWORD specifies the wave file buffer size in
//							number of samples.  
//		SampleLimit = limit on number of samples to be written.
//		ChainFiles	= 1(TRUE) means to Automatically close the current file
//                    when it reaches sample limit then open a new one with
//                    a timestamp appended to the name and contimue writing.
//					  (FALSE) means just use the given file.
//    returns:
//        0			if opened OK
//    ErrorCode		if not
//////////////////////////////////////////////////////////////////////
UINT CWave::OutOpen( CString* pPathName, WAVEFORMATEX* pWFX, 
						DWORD BufSize, DWORD SampleLimit, 
						INT ChainFiles, AUXINFO* pAuxInfo)
{
	m_OutPath = *pPathName;
	m_OutFormat = *pWFX;
	m_BufferSize = BufSize;
	m_OutSampleLimit = SampleLimit;
	m_ChainFiles = ChainFiles;
	if(pAuxInfo)
	{
		memcpy(&m_OutAuxInfo,pAuxInfo,sizeof(AUXINFO));
		m_UseAuxStructure = TRUE;
	}
	else
	{
		m_UseAuxStructure = FALSE;
	}
//
	m_OutputOpen = FALSE;
	m_ErrorCode = NO_ERRORS;
	m_pOutBuffer = NULL;
	m_OutBufPosition = 0;
	m_OutSamplesWritten = 0;
	m_AuxPosition = 0;
	memset( &m_outmmcktRIFF,0,sizeof(MMCKINFO));
	memset( &m_outmmcktSubchunk,0,sizeof(MMCKINFO));
	m_OutBytesPerSample = (m_OutFormat.wBitsPerSample/8)*m_OutFormat.nChannels;
	m_OutBufferSize = BufSize * m_OutBytesPerSample;
	CString sPath = *pPathName;
	CString sSfx = sPath.Right(4);	//get last 4 characters of filepath(should just be ".wav")
	CString str;
	if(ChainFiles & FILE_CHAIN_FREQSTAMP)	//need to add center freq to filename
	{
		if( (8000==m_OutFormat.nSamplesPerSec) || (48000==m_OutFormat.nSamplesPerSec) )	//is demod output
			str.Format("-D%luHz.wav", m_OutAuxInfo.CenterFreq);
 //			str = "-DemodFile.wav";
		else
			str.Format("-%luHz.wav", m_OutAuxInfo.CenterFreq);
		sPath.Replace(sSfx, str );
	}
	if( ChainFiles & FILE_CHAIN_TIMESTAMP )	//need to add timestamp to filename
	{
		sSfx = sPath.Right(4);
		CTime time = CTime::GetCurrentTime();
		sPath.Replace(sSfx, time.Format(_T("_%Y%m%d_%H%M%S.wav")) );
	}
	else
	{	//if no timestamp
		if( ChainFiles & FILE_CHAIN )	//but needs to chain so adds sequence numbers
		{
			sSfx = sPath.Right(4);
			m_ChainCount++;
			m_Str.Format("_%3.3d.wav",m_ChainCount);
			sPath.Replace(sSfx, m_Str );
		}
	}


// Create the output file RIFF chunk of form type 'WAVE'.
	m_OUThmmio = mmioOpen( (LPSTR)LPCTSTR(sPath) , NULL,
								MMIO_WRITE|MMIO_ALLOCBUF|MMIO_CREATE );
	if( m_OUThmmio == NULL)
	{
		m_ErrorCode = WAVOUT_ERR_OPEN;
		return m_ErrorCode;
	}
	m_outmmcktRIFF.fccType = mmioFOURCC('W', 'A', 'V', 'E');
//		m_outmmcktRIFF.cksize = 4;
	if( mmioCreateChunk(m_OUThmmio, &m_outmmcktRIFF, MMIO_CREATERIFF) != 0)
	{
		mmioClose(m_OUThmmio,0);
		m_ErrorCode = WAVOUT_ERR_WRITING;
		return m_ErrorCode;
	}
	// Are now descended into the 'RIFF' chunk just created.
	// Now create the 'fmt ' chunk. Since the size of this chunk is known,
	// specify it in the MMCKINFO structure so MMIO doesn't have to seek
	// back and set the chunk size after ascending from the chunk.
	m_outmmcktSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
	m_outmmcktSubchunk.cksize = sizeof(WAVEFORMATEX);  // we know the size of this ck.
	if( mmioCreateChunk(m_OUThmmio, &m_outmmcktSubchunk, 0) != 0)
	{
		mmioClose(m_OUThmmio,0);
		m_ErrorCode = WAVOUT_ERR_WRITING;
		return m_ErrorCode;
	}
	// Write the WAVEFORMATEX structure to the 'fmt ' chunk.
	if( mmioWrite(m_OUThmmio, (HPSTR)pWFX, sizeof(WAVEFORMATEX))
				!= sizeof(WAVEFORMATEX))
	{
		mmioClose(m_OUThmmio,0);
		m_ErrorCode = WAVOUT_ERR_WRITING;
		return m_ErrorCode;
	}
	// Ascend out of the 'fmt ' chunk, back into the 'RIFF' chunk.
	if( mmioAscend(m_OUThmmio, &m_outmmcktSubchunk, 0) != 0)
	{
		mmioClose(m_OUThmmio,0);
		m_ErrorCode = WAVOUT_ERR_WRITING;
		return m_ErrorCode;
	}
	if(m_UseAuxStructure)	//create an AUXI chunk if struct not NULL
	{
		m_outmmcktSubchunk.ckid = mmioFOURCC('a', 'u', 'x', 'i');
		m_outmmcktSubchunk.cksize = sizeof(AUXINFO);  // we know the size of this ck.
		if( mmioCreateChunk(m_OUThmmio, &m_outmmcktSubchunk, 0) != 0)
		{
			mmioClose(m_OUThmmio,0);
			m_ErrorCode = WAVOUT_ERR_WRITING;
			return m_ErrorCode;
		}
		// Write the AUXINFO structure to the 'auxi' chunk.
		m_AuxPosition = m_outmmcktSubchunk.dwDataOffset;
		if( mmioWrite(m_OUThmmio, (HPSTR)&m_OutAuxInfo, sizeof(AUXINFO))
						!= sizeof(AUXINFO))
		{
			mmioClose(m_OUThmmio,0);
			m_ErrorCode = WAVOUT_ERR_WRITING;
			return m_ErrorCode;
		}
		// Ascend out of the 'auxi' chunk, back into the 'RIFF' chunk.
		if( mmioAscend(m_OUThmmio, &m_outmmcktSubchunk, 0) != 0)
		{
			mmioClose(m_OUThmmio,0);
			m_ErrorCode = WAVOUT_ERR_WRITING;
			return m_ErrorCode;
		}
	}
	// Create the 'data' chunk that holds the waveform samples.
	m_outmmcktSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
	if (mmioCreateChunk(m_OUThmmio, &m_outmmcktSubchunk, 0) != 0)
	{
		mmioClose(m_OUThmmio,0);
		m_ErrorCode = WAVOUT_ERR_WRITING;
		return m_ErrorCode;
	}
	if( m_ErrorCode == NO_ERRORS )
	{
		// allocate intermediate byte buffer
		m_pOutBuffer = new char[ m_OutBufferSize ];
		if( m_pOutBuffer == NULL )
		{
			OutClose();
			m_ErrorCode = MEMORY_ERROR;
		}
		else
		{
			m_OutputOpen = TRUE;
		}
	}
	return m_ErrorCode;
}

///////////////////////////////////////////////////////////////////////
// Writes 'Length' shorts to the open wavefile.
//    parameters:
//		pData	= pointer to block of 'Length' SHORT INT's to output.
//		Length	= Number of samples to write from pData. If is zero
//					then the output file is closed.
//
// Returns:
//		'Length' if data was succesfully placed in the output buffers.
//       0 if output has finished( reached the specified sample limit )
//      -1		 if error ( use GetError() to retrieve error )
//      -2		 if End of chained file
///////////////////////////////////////////////////////////////////////
LONG CWave::OutWrite( SHORT* pData, INT Length)
{
INT i;
	if( !m_OutputOpen )
	{
		OutClose();
		m_ErrorCode = WAVOUT_ERR_NOTOPEN;
		return -1;
	}
	if( Length == 0 )
	{
		if( mmioWrite( m_OUThmmio, m_pOutBuffer, m_OutBufPosition ) < 0 )
		{
			OutClose();
			m_ErrorCode = WAVOUT_ERR_WRITING;
			return -1;
		}
		OutClose();
		return 0;
	}
	if(m_OutFormat.wBitsPerSample == 16)
	{
		for( i=0; i < (Length*m_OutFormat.nChannels); i++ )
		{
			m_usTemp.both = (SHORT)(*( pData + i));
			*(m_pOutBuffer + (m_OutBufPosition++)) = m_usTemp.bytes.lsb;
			*(m_pOutBuffer + (m_OutBufPosition++)) = m_usTemp.bytes.msb;
		}
	}
	else
	{
		for( i=0; i < (Length*m_OutFormat.nChannels); i++ )
		{
			m_usTemp.both = (SHORT)(*(pData + i) + 32768.0);
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_usTemp.bytes.msb;
		}
	}
	if( mmioWrite( m_OUThmmio, m_pOutBuffer, m_OutBufPosition ) < 0 )
	{
		OutClose();
		m_ErrorCode = WAVOUT_ERR_WRITING;
		return -1;
	}
	m_OutBufPosition = 0;	// need to write another buffer's worth
	m_OutSamplesWritten += Length;
	if(( (INT)m_OutSampleLimit >0 ) && (m_OutSamplesWritten >= m_OutSampleLimit) )
	{
		OutClose();
		if( m_ChainFiles & FILE_CHAIN )
		{
			CTime ctime;
			ctime = CTime::GetCurrentTime();
			ctime.GetAsSystemTime( m_OutAuxInfo.StartTime );
			m_OutAuxInfo.StopTime = m_OutAuxInfo.StartTime;
			if( NO_ERROR == OutOpen( &m_OutPath, &m_OutFormat, m_BufferSize,
						m_OutSampleLimit, m_ChainFiles, &m_OutAuxInfo) )
			{
				return -2;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			return 0;
		}
	}
	return Length;
}

///////////////////////////////////////////////////////////////////////
// Writes 'Length' ints to the open wavefile.(int can be 32 or 24 bit)
//    parameters:
//		pData	= pointer to block of 'Length' SHORT INT's to output.
//		Length	= Number of samples to write from pData. If is zero
//					then the output file is closed.
//
// Returns:
//		'Length' if data was succesfully placed in the output buffers.
//       0 if output has finished( reached the specified sample limit )
//      -1		 if error ( use GetError() to retrieve error )
//      -2		 if End of chained file
///////////////////////////////////////////////////////////////////////
LONG CWave::OutWrite( _int32* pData, INT Length)
{
INT i;
	if( !m_OutputOpen )
	{
		OutClose();
		m_ErrorCode = WAVOUT_ERR_NOTOPEN;
		return -1;
	}
	if( Length == 0 )
	{
		if( mmioWrite( m_OUThmmio, m_pOutBuffer, m_OutBufPosition ) < 0 )
		{
			OutClose();
			m_ErrorCode = WAVOUT_ERR_WRITING;
			return -1;
		}
		OutClose();
		return 0;
	}
	if(m_OutFormat.wBitsPerSample == 24)
	{
		for( i=0; i < (Length*m_OutFormat.nChannels); i++ )
		{
			m_uiTemp.all = pData[i];
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b1;
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b2;
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b3;
		}
	}
	else if(m_OutFormat.wBitsPerSample == 16)
	{
		for( i=0; i < (Length*m_OutFormat.nChannels); i++ )
		{
			m_uiTemp.all = pData[i];
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b2;
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b3;
		}
	}
	else if(m_OutFormat.wBitsPerSample == 32)
	{
		for( i=0; i < (Length*m_OutFormat.nChannels); i++ )
		{
			m_uiTemp.all = pData[i];
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b0;
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b1;
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b2;
			*(m_pOutBuffer + (m_OutBufPosition++)) = (char)m_uiTemp.bytes.b3;
		}
	}
	if( mmioWrite( m_OUThmmio, m_pOutBuffer, m_OutBufPosition ) < 0 )
	{
		OutClose();
		m_ErrorCode = WAVOUT_ERR_WRITING;
		return -1;
	}
	m_OutBufPosition = 0;	// need to write another buffer's worth
	m_OutSamplesWritten += Length;
	if(( (INT)m_OutSampleLimit >0 ) && (m_OutSamplesWritten >= m_OutSampleLimit) )
	{
		OutClose();
		if( m_ChainFiles & FILE_CHAIN )
		{
			CTime ctime;
			ctime = CTime::GetCurrentTime();
			ctime.GetAsSystemTime( m_OutAuxInfo.StartTime );
			m_OutAuxInfo.StopTime = m_OutAuxInfo.StartTime;
			if( NO_ERROR == OutOpen( &m_OutPath, &m_OutFormat, m_BufferSize,
						m_OutSampleLimit, m_ChainFiles, &m_OutAuxInfo) )
			{
				return -2;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			return 0;
		}
	}
	return Length;
}

void CWave::UseOpenTimestamp()
{
	m_AuxPosition = 0;
}

//////////////////////////////////////////////////////////////////////
//  Change IF Frequency parameter in m_OutAuxInfo while file is open
//////////////////////////////////////////////////////////////////////
void CWave::ChangeOutFileParameters(int IFFreq)
{
	if( m_OutputOpen )
		m_OutAuxInfo.IFFrequency = IFFreq;
}

//////////////////////////////////////////////////////////////////////
//  Closes the Output wave file by first updating the information chunk
//    with the length of data that was written then closing it.
//////////////////////////////////////////////////////////////////////
void CWave::OutClose()
{
INT x;   
CTime ctime;
	if( m_OUThmmio )
	{
		if( m_OutputOpen )	// need to update file and data lengths in
		{					// the wave header if the file was opened
	   		if( (x=mmioWrite( m_OUThmmio, m_pOutBuffer, m_OutBufPosition )) >= 0 )
			{
				// Ascend out of the 'data' chunk, back into the 'RIFF' chunk.
				if( mmioAscend(m_OUThmmio, &m_outmmcktSubchunk, 0) == 0)
				{	
					LONG pos = mmioSeek( m_OUThmmio, 0, SEEK_CUR );
					if(m_AuxPosition)
					{	// Write the AUXINFO structure to the 'auxi' chunk.
						mmioSeek( m_OUThmmio, m_AuxPosition, SEEK_SET );
						SYSTEMTIME stime;  // holds current time
						::GetSystemTime(&stime);	//get current UTC time
						ctime = stime;
						ctime.GetAsSystemTime( m_OutAuxInfo.StopTime );
						if( mmioWrite(m_OUThmmio, (HPSTR)&m_OutAuxInfo, sizeof(AUXINFO))
									!= sizeof(AUXINFO))
						{
							mmioClose(m_OUThmmio,0);
							m_ErrorCode = WAVOUT_ERR_WRITING;
						}
					}
					mmioSeek( m_OUThmmio, pos, SEEK_SET );
					// Ascend out of the 'RIFF' chunk, writing the RIFF file length.
					int x=mmioAscend(m_OUThmmio, &m_outmmcktRIFF, 0);
				}
			}
	   }
		mmioClose( m_OUThmmio, 0 );
		m_OUThmmio = FALSE;
	}
	if( m_pOutBuffer )
	{
		delete m_pOutBuffer;
		m_pOutBuffer = NULL;
	}
	m_OutputOpen = FALSE;
}


/////////////////////////////////////////////////////////////////
// returns free disk space on drive specified by pPathName.
// The extra logic is for dealing with early Win95 versions that don't
// support the extended disk space function "GetDiskFreeSpaceEx(..)".
/////////////////////////////////////////////////////////////////
LONGLONG CWave::GetDriveFreeSpace(CString* pPathName)
{
typedef BOOL (CALLBACK* LPFNDLLFUNC)(LPCTSTR, PULARGE_INTEGER,
					 PULARGE_INTEGER, PULARGE_INTEGER);

LONGLONG SpaceFree;
	SpaceFree = 0;					// see if it exists
	HMODULE hKernel = GetModuleHandle( _T("kernel32.dll"));
	LPFNDLLFUNC pFunct = (LPFNDLLFUNC)GetProcAddress( hKernel,"GetDiskFreeSpaceExA");
	if(pFunct != NULL)		//if function is in kernel32.dll
	{
		ULARGE_INTEGER TotalSpace;
		ULARGE_INTEGER FreeSpace;
		pFunct( pPathName->Left(3), &FreeSpace, &TotalSpace, NULL);
		SpaceFree = FreeSpace.QuadPart;
	}
	else					//else is WIn95 pre OSR2 so use old function
	{
		DWORD SectorsPerCluster, BytesPerSector;
		DWORD NumberOfFreeClusters, TotalNumberOfClusters;
		if( GetDiskFreeSpace( pPathName->Left(3),
					&SectorsPerCluster, &BytesPerSector,
					&NumberOfFreeClusters, &TotalNumberOfClusters ) ) 
		SpaceFree = (LONGLONG)(NumberOfFreeClusters*SectorsPerCluster*BytesPerSector);
	}
	return SpaceFree;
}



/////////////////////////////////////////////////////////////////
// Converts pSlocal from local time to UTC time.
/////////////////////////////////////////////////////////////////
SYSTEMTIME CWave::ConvertLocalToUTC(SYSTEMTIME* pSlocal)
{
SYSTEMTIME stime;
struct tm t_tm;
CTime ltime = *pSlocal;		//create CTime object from local time
	ltime.GetLocalTm(&t_tm);	//convert to local tm structure to get daylight savings mode
	int dst = t_tm.tm_isdst;
	ltime.GetGmtTm(&t_tm);	//convert to UTC tm structure
	stime.wDay = t_tm.tm_wday;
	stime.wDayOfWeek = t_tm.tm_wday;
	stime.wHour = t_tm.tm_hour - dst;
	stime.wMinute = t_tm.tm_min;
	stime.wMonth = t_tm.tm_mon+1;
	stime.wSecond = t_tm.tm_sec;
	stime.wYear = t_tm.tm_year + 1900;
	stime.wMilliseconds = pSlocal->wMilliseconds;
	stime.wDay = t_tm.tm_mday;
	return stime;
}
